package eu.DECIDEh2020.architect.plugin.builder;

import java.io.File;
import java.io.IOException;
import java.util.Map;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.jgit.api.errors.GitAPIException;
import org.eclipse.jgit.lib.Ref;
import org.eclipse.jgit.lib.Repository;
import org.eclipse.jgit.storage.file.FileRepositoryBuilder;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import eu.DECIDEh2020.appManager.persistence.GitHandler;
import eu.DECIDEh2020.architect.plugin.Activator;

public class DecideBuilder extends IncrementalProjectBuilder {

    public static final String BUILDER_ID = "eu.DECIDEh2020.architect.plugin.DecideBuilder";
    public static final String DECIDE_FILENAME = "DECIDE.json";

    public DecideBuilder() {
    }

    @Override
    protected IProject[] build(int kind, Map<String, String> args, IProgressMonitor monitor) throws CoreException {
	// get the project to build
	IProject project = getProject();

	switch (kind) {

	case FULL_BUILD:
	    try {
		monitor.beginTask("Adding Nature", 3);

		boolean isGit = false;

		IPath projectLocation = project.getLocation();
		FileRepositoryBuilder builder = new FileRepositoryBuilder();
		File dir = projectLocation.toFile();
		try (Repository repository = builder.setGitDir(dir).readEnvironment() // scan environment GIT_* variables
			.findGitDir() // scan up the file system tree
			.build()) {
		    // System.out.println("Having repository: " + repository.getDirectory());

		    // the Ref holds an ObjectId for any type of object (tree, commit, blob, tree)
		    Ref head = repository.exactRef("refs/heads/master");
		    if (head != null) {
			isGit = true;
		    }

		} catch (IOException e) {
		    isGit = false;
		    Activator.getDefault().getLog().log(new Status(Status.ERROR, Activator.PLUGIN_ID, "Check for git.", e));
		}

		monitor.subTask("Adding Git Directory");

		if (!isGit) {
		    GitHandler gh = new GitHandler(new GitHandler.Repository());
		    gh.createRepositoryIn(projectLocation.toFile().toPath());
		}
		monitor.worked(1);
		boolean isDECIDE = false;
		File decidejson = new File(projectLocation + File.separator + "DECIDE.json");
		if (decidejson.exists() && !decidejson.isDirectory()) {
		    isDECIDE = true;
		}
		// System.out.println("IS DECIDE: " + isDECIDE);

		if (!isDECIDE) {
		    monitor.subTask("Wizard");
		    BuilderWizard bwizard = new BuilderWizard(monitor, project);
		    PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
			public void run() {
			    Shell activeShell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
			    WizardDialog dialog = new WizardDialog(activeShell, bwizard);
			    dialog.open();
			}
		    });
		    monitor.worked(1);
		}

	    } catch (IllegalStateException e) {
		Activator.getDefault().getLog().log(new Status(Status.ERROR, Activator.PLUGIN_ID, "Full build.", e));
	    } catch (GitAPIException e) {
		Activator.getDefault().getLog().log(new Status(Status.ERROR, Activator.PLUGIN_ID, "Full build.", e));
	    } finally {
		monitor.done();
	    }
	    break;

	case INCREMENTAL_BUILD:
	    break;

	case AUTO_BUILD:
	    break;
	}
	return null;
    }

}
