/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* Eclipse Public License version 2.0 which accompanies
* this distribution, and is available at
* https://opensource.org/licenses/EPL-2.0
*
* Contributors:
*
* Torben Jastrow (Fraunhofer FOKUS) 
* 
* **Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu */

package eu.DECIDEh2020.architect.plugin.descriptorWizard;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.custom.ScrolledComposite;

import eu.DECIDEh2020.appManager.models.AppDescription;
import eu.DECIDEh2020.appManager.models.Microservice;
import eu.DECIDEh2020.architect.plugin.layoutComponents.MicroserviceComposite;

import java.util.List;

//Wizard page that contains a list of all microservices & functions to change their properties
public class MicroservicePage extends WizardPage {

    private ScrolledComposite scrolledComposite;
    private Composite container;
    private AppDescription project;

    public MicroservicePage(AppDescription project) {
	super("Microservice Settings");
	setTitle("Microservice Settings");
	setDescription("Configure each Microservice here");
	this.project = project;
    }

    @Override
    public void createControl(Composite parent) {

	scrolledComposite = new ScrolledComposite(parent, SWT.V_SCROLL);
	scrolledComposite.setExpandHorizontal(true);
	scrolledComposite.setExpandVertical(true);
	setControl(scrolledComposite);
	container = new Composite(scrolledComposite, SWT.NONE);
	scrolledComposite.setContent(container);
	container.setLayout(new GridLayout(1, false));
	scrolledComposite.setExpandVertical(true);
	scrolledComposite.setExpandHorizontal(true);
	scrolledComposite.addListener(SWT.Resize, event -> {
	    int width = scrolledComposite.getClientArea().width;
	    scrolledComposite.setMinSize(container.computeSize(width, SWT.DEFAULT));
	});

	setPageComplete(true);
    }

    private void addMicroserviceComposite(Microservice microservice) {
	if (microservice != null) {
	    @SuppressWarnings("unused")
	    Composite msvContainer = new MicroserviceComposite(container, microservice, project);
	}
    }

    public void setMicroservices() {
	// first remove all microservice composites & then add new ones
	Control[] children = container.getChildren();
	for (int i = 0; i < children.length; i++) {
	    children[i].dispose();
	}

	addMicroservices();
	container.requestLayout();
	int width = scrolledComposite.getClientArea().width;
	scrolledComposite.setMinSize(container.computeSize(width, SWT.DEFAULT));
	scrolledComposite.requestLayout();

    }

    private void addMicroservices() {

	List<Microservice> microservices = project.getMicroservices();
	if (microservices.size() > 0) {
	    for (Microservice m : microservices) {
		addMicroserviceComposite(m);
	    }

	}

    }
}
