/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* Eclipse Public License version 2.0 which accompanies
* this distribution, and is available at
* https://opensource.org/licenses/EPL-2.0
*
* Contributors:
*
* Torben Jastrow (Fraunhofer FOKUS) 
* 
* **Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu */
package eu.DECIDEh2020.architect.plugin.descriptorWizard;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.dialogs.WizardNewProjectCreationPage;

import eu.DECIDEh2020.appManager.models.AppDescription;
import eu.DECIDEh2020.appManager.persistence.GitHandler.Repository;

import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;

import java.net.URISyntaxException;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.jface.databinding.swt.ISWTObservableValue;
import org.eclipse.jface.databinding.swt.SWTObservables;
import org.eclipse.jface.databinding.swt.WidgetProperties;
import org.eclipse.jgit.transport.URIish;
import org.eclipse.core.databinding.beans.PojoProperties;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;

public class NewProjectPage extends WizardNewProjectCreationPage {

    private Composite container;
    private AppDescription project;
    private Repository repo;
    private Text txtUser;
    private Text txtPassword;
    private Text txtPort;
    private Text txtReponame;
    private Text txtUri;
    private Text txtHost;
    private Text txtRepopath;
    private Combo comboProtocol;
    private Button btnSecurestore;
    private Button btnCreateLocalRepository;
    private Button btnOpenLocalRepository;
    private Button btnCloneRemoteRepository;

    public NewProjectPage(String pageName, AppDescription project) {
	super(pageName);
	this.project = project;
	repo = new Repository();

    }

    @Override
    public void createControl(Composite parent) {
	super.createControl(parent);
	Composite p = (Composite) getControl();

	p.setLayout(new GridLayout(1, false));
	container = new Composite(p, SWT.NONE);
	container.setLayout(new GridLayout(1, false));
	container.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));

	Composite compositeLocal_Remote = new Composite(container, SWT.NONE);
	compositeLocal_Remote.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, false, 1, 1));
	compositeLocal_Remote.setLayout(new FillLayout(SWT.HORIZONTAL));

	btnOpenLocalRepository = new Button(compositeLocal_Remote, SWT.RADIO);
	btnOpenLocalRepository.setText("Open Local repository");

	btnCreateLocalRepository = new Button(compositeLocal_Remote, SWT.RADIO);

	btnCreateLocalRepository.setText("Create Local repository");

	btnCloneRemoteRepository = new Button(compositeLocal_Remote, SWT.RADIO);

	btnCloneRemoteRepository.setText("Clone Remote Repository");

	Group grpGitRepository = new Group(container, SWT.NONE);
	grpGitRepository.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
	grpGitRepository.setText("Git Repository");
	grpGitRepository.setLayout(new GridLayout(1, false));

	Composite composite_1 = new Composite(grpGitRepository, SWT.NONE);
	composite_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
	composite_1.setLayout(new GridLayout(2, false));

	Label lblName = new Label(composite_1, SWT.NONE);
	lblName.setText("Name:");

	txtReponame = new Text(composite_1, SWT.BORDER);
	txtReponame.setEnabled(false);
	txtReponame.addKeyListener(new KeyAdapter() {
	    @Override
	    public void keyPressed(KeyEvent e) {

	    }
	});
	txtReponame.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

	Composite composite = new Composite(grpGitRepository, SWT.NONE);
	composite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
	composite.setLayout(new GridLayout(1, false));

	Group grpLocation = new Group(composite, SWT.NONE);
	grpLocation.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
	grpLocation.setText("Location");
	grpLocation.setLayout(new GridLayout(2, false));

	Label lblUri = new Label(grpLocation, SWT.NONE);
	lblUri.setText("URI:");

	txtUri = new Text(grpLocation, SWT.BORDER);

	txtUri.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

	Label lblHost = new Label(grpLocation, SWT.NONE);
	lblHost.setText("Host:");

	txtHost = new Text(grpLocation, SWT.BORDER);
	txtHost.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

	Label lblRepositoryPath = new Label(grpLocation, SWT.NONE);
	lblRepositoryPath.setText("Repository Path:");

	txtRepopath = new Text(grpLocation, SWT.BORDER);
	txtRepopath.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

	Group grpConnection = new Group(composite, SWT.NONE);
	grpConnection.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
	grpConnection.setText("Connection");
	grpConnection.setLayout(new GridLayout(2, false));

	Label lblProtocol = new Label(grpConnection, SWT.NONE);
	lblProtocol.setText("Protocol:");

	comboProtocol = new Combo(grpConnection, SWT.READ_ONLY);
	comboProtocol.setItems(new String[] { "https", "ssh" });
	comboProtocol.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, false, 1, 1));
	comboProtocol.select(0);

	Label lblPort = new Label(grpConnection, SWT.NONE);
	lblPort.setText("Port:");

	txtPort = new Text(grpConnection, SWT.BORDER);
	txtPort.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, false, 1, 1));

	Group grpAuthentication = new Group(composite, SWT.NONE);
	grpAuthentication.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
	grpAuthentication.setText("Authentication");
	grpAuthentication.setLayout(new GridLayout(2, false));

	Label lblUser = new Label(grpAuthentication, SWT.NONE);
	lblUser.setText("User:");

	txtUser = new Text(grpAuthentication, SWT.BORDER);
	txtUser.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

	Label lblPassword = new Label(grpAuthentication, SWT.NONE);
	lblPassword.setText("Password:");

	txtPassword = new Text(grpAuthentication, SWT.BORDER | SWT.PASSWORD);
	txtPassword.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

	Label lblStoreInSecure = new Label(grpAuthentication, SWT.NONE);
	lblStoreInSecure.setText("Store in Secure Store:");

	txtUri.addKeyListener(new KeyAdapter() {
	    @Override
	    public void keyReleased(KeyEvent e) {
		URIish uri = null;
		try {
		    if (!txtUri.getText().isEmpty()) {
			uri = new URIish(txtUri.getText());
		    }
		} catch (URISyntaxException e1) {
		    e1.printStackTrace();
		}
		if (uri != null) {
		    txtHost.setText(uri.getHost() != null ? uri.getHost() : "");
		    txtRepopath.setText(uri.getPath() != null ? uri.getPath() : "");

		    comboProtocol.setText(uri.getScheme() != null ? uri.getScheme() : "ssh");
		    txtUser.setText(uri.getUser() != null ? uri.getUser() : "");
		    txtPassword.setText(uri.getPass() != null ? uri.getPass() : "");

		    if (uri.getPort() >= 0) {
			txtPort.setText(String.valueOf(uri.getPort()));
		    }

		}
		setPageComplete(true);
	    }
	});

	btnSecurestore = new Button(grpAuthentication, SWT.CHECK);
	txtUri.setEnabled(false);
	txtHost.setEnabled(false);
	txtRepopath.setEnabled(false);
	comboProtocol.setEnabled(false);
	txtPort.setEnabled(false);
	txtUser.setEnabled(false);
	txtPassword.setEnabled(false);
	btnSecurestore.setEnabled(false);

	btnOpenLocalRepository.addSelectionListener(new SelectionAdapter() {
	    @Override
	    public void widgetSelected(SelectionEvent e) {
		txtReponame.setEnabled(false);
		txtUri.setEnabled(false);
		txtHost.setEnabled(false);
		txtRepopath.setEnabled(false);
		comboProtocol.setEnabled(false);
		txtPort.setEnabled(false);
		txtUser.setEnabled(false);
		txtPassword.setEnabled(false);
		btnSecurestore.setEnabled(false);
		repo.setNewRepo(false);
		repo.setLocalRepo(true);

		setPageComplete(true);
		getWizard().getContainer().updateButtons();
		getWizard().getContainer().updateButtons();

	    }
	});

	btnCreateLocalRepository.addSelectionListener(new SelectionAdapter() {
	    @Override
	    public void widgetSelected(SelectionEvent e) {

		txtReponame.setEnabled(false);
		txtUri.setEnabled(false);
		txtHost.setEnabled(false);
		txtRepopath.setEnabled(false);
		comboProtocol.setEnabled(false);
		txtPort.setEnabled(false);
		txtUser.setEnabled(false);
		txtPassword.setEnabled(false);
		btnSecurestore.setEnabled(false);
		repo.setNewRepo(true);

		setPageComplete(true);
		getWizard().getContainer().updateButtons();
		getWizard().getContainer().updateButtons();
	    }
	});

	btnCloneRemoteRepository.addSelectionListener(new SelectionAdapter() {
	    @Override
	    public void widgetSelected(SelectionEvent e) {
		txtReponame.setEnabled(false);
		txtUri.setEnabled(true);
		txtHost.setEnabled(true);
		txtRepopath.setEnabled(true);
		comboProtocol.setEnabled(true);
		txtPort.setEnabled(true);
		txtUser.setEnabled(true);
		txtPassword.setEnabled(true);
		btnSecurestore.setEnabled(true);
		repo.setNewRepo(false);
		repo.setLocalRepo(false);

		setPageComplete(true);
		getWizard().getContainer().updateButtons();
		getWizard().getContainer().updateButtons();
	    }
	});
	btnCreateLocalRepository.setSelection(true);
	setPageComplete(false);
	// required to avoid an error in the system
	setControl(p);
	initDataBindings();

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.wizard.WizardPage#setPageComplete(boolean)
     */
    @Override
    public void setPageComplete(boolean complete) {
	Boolean thisComplete = false;
	if ((btnOpenLocalRepository != null && btnOpenLocalRepository.getSelection())
		|| (btnCreateLocalRepository != null && btnCreateLocalRepository.getSelection())) {
	    thisComplete = true;
	} else {
	    if (repo != null && repo.getURI() != null && !repo.getURI().isEmpty()) {
		thisComplete = true;
	    }
	}
	thisComplete = thisComplete && this.getProjectName() != null && this.getProjectName() != "" && getErrorMessage() == null;

	super.setPageComplete(thisComplete && complete);
	if (getWizard() != null && getWizard().getContainer() != null) {
	    getWizard().getContainer().updateButtons();
	}
	// System.out.println("NewProject Page Complete: "+ isPageComplete());
    }

    public boolean isPageComplete() {
	if (project != null) {
	    project.setName(getProjectName());
	}
	return super.isPageComplete();
    }

    public Repository getRepository() {
	return repo;
    }

    protected DataBindingContext initDataBindings() {
	DataBindingContext bindingContext = new DataBindingContext();
	//
	ISWTObservableValue observeTextTxtReponameObserveWidget = WidgetProperties.text(SWT.Modify).observe(txtReponame);
	IObservableValue nameRepoObserveValue = PojoProperties.value("name").observe(repo);
	bindingContext.bindValue(SWTObservables.observeText(txtReponame, SWT.Modify), nameRepoObserveValue);
	//
	IObservableValue<String> observeTextTxtUriObserveWidget = WidgetProperties.text(SWT.Modify).observe(txtUri);
	IObservableValue uRIRepoObserveValue = PojoProperties.value("URI").observe(repo);
	bindingContext.bindValue(observeTextTxtUriObserveWidget, uRIRepoObserveValue);
	//
	IObservableValue<String> observeTextTxtHostObserveWidget = WidgetProperties.text(SWT.Modify).observe(txtHost);
	IObservableValue<String> hostRepoObserveValue = PojoProperties.value("host").observe(repo);
	bindingContext.bindValue(observeTextTxtHostObserveWidget, hostRepoObserveValue);
	//
	IObservableValue<String> observeTextTxtRepopathObserveWidget = WidgetProperties.text(SWT.Modify).observe(txtRepopath);
	IObservableValue<String> pathRepoObserveValue = PojoProperties.value("path").observe(repo);
	bindingContext.bindValue(observeTextTxtRepopathObserveWidget, pathRepoObserveValue);
	//
	IObservableValue<String> observeTextComboProtocolObserveWidget = WidgetProperties.text().observe(comboProtocol);
	IObservableValue<String> protocolRepoObserveValue = PojoProperties.value("protocol").observe(repo);
	bindingContext.bindValue(observeTextComboProtocolObserveWidget, protocolRepoObserveValue);
	//
	IObservableValue<String> observeTextTextPortObserveWidget = WidgetProperties.text(SWT.Modify).observe(txtPort);
	IObservableValue<String> portRepoObserveValue = PojoProperties.value("port").observe(repo);
	bindingContext.bindValue(observeTextTextPortObserveWidget, portRepoObserveValue);
	//
	IObservableValue<String> observeTextTxtUserObserveWidget = WidgetProperties.text(SWT.Modify).observe(txtUser);
	IObservableValue<String> userRepoObserveValue = PojoProperties.value("user").observe(repo);
	bindingContext.bindValue(observeTextTxtUserObserveWidget, userRepoObserveValue);
	//
	IObservableValue<String> observeTextTxtPasswordObserveWidget = WidgetProperties.text(SWT.Modify).observe(txtPassword);
	IObservableValue<String> passwordRepoObserveValue = PojoProperties.value("password").observe(repo);
	bindingContext.bindValue(observeTextTxtPasswordObserveWidget, passwordRepoObserveValue);
	//
	IObservableValue<String> observeSelectionBtnSecurestoreObserveWidget = WidgetProperties.selection().observe(btnSecurestore);
	IObservableValue<String> secureStorageRepoObserveValue = PojoProperties.value("secureStorage").observe(repo);
	bindingContext.bindValue(observeSelectionBtnSecurestoreObserveWidget, secureStorageRepoObserveValue);
	//
	return bindingContext;
    }
}
