/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* Eclipse Public License version 2.0 which accompanies
* this distribution, and is available at
* https://opensource.org/licenses/EPL-2.0
*
* Contributors:
*
* Torben Jastrow (Fraunhofer FOKUS) 
* 
* **Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu */
package eu.DECIDEh2020.architect.plugin.descriptorWizard;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.core.runtime.*;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ISelection;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.lang.reflect.InvocationTargetException;
import java.net.URI;

import org.eclipse.ui.*;

import eu.DECIDEh2020.appManager.AppDescriptionFactory;
import eu.DECIDEh2020.appManager.models.AppDescription;
import eu.DECIDEh2020.appManager.persistence.GitHandler.Repository;



/**
 * This is a decide new wizard. Its role is to create a new project and either 
 * clone or create a new appdescription file while asking for infos how that 
 * appdescription should be initialized
 * It will then open the decide multi-page editor with the created decide.json
 * 
 */

public class NewProjectWizard extends Wizard implements INewWizard, IExecutableExtension {
	@SuppressWarnings("unused")
	private ISelection selection;
	private IConfigurationElement _configurationElement;
	private MicroservicePage microservicePage;
	private NewProjectPage projectPage;
	private ConfigPage configPage;
	private MonolithPage monolithPage;
	private AppDescription project;

	/**
	 * Constructor for NewProjectWizard.
	 */
	public NewProjectWizard() {
		super();
		setNeedsProgressMonitor(true);
		setWindowTitle("DECIDE project wizard");
		try {
			project= getProject();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public AppDescription getProject() {
		if (project==null) {
			try {
				return AppDescriptionFactory.createAppDescription("DECIDE");
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return project;
	}

	/**
	 * Adding the pages to the wizard.
	 */

	public void addPages() {
		super.addPages();

		projectPage = new NewProjectPage("From Scratch Project Wizard", project);
		projectPage.setTitle("New DECIDE Project");
		projectPage.setDescription("Create a new DECIDE project.");
		addPage(projectPage);

		configPage = new ConfigPage(project);
		addPage(configPage);

		microservicePage = new MicroservicePage(project);
		addPage(microservicePage);
		monolithPage = new MonolithPage();
		addPage(monolithPage);
	}

	/**
	 * This method is called when 'Finish' button is pressed in the wizard. We will
	 * create an operation and run it using wizard as execution context.
	 */
	public boolean performFinish() {

		final String name = projectPage.getProjectName();
		URI loc = null;
		if (!projectPage.useDefaults()) {
			loc = projectPage.getLocationURI();
			System.err.println("location: " + loc.toString()); //$NON-NLS-1$
		} // else location == null
		final URI location = loc;
		final Repository repo = projectPage.getRepository();
		
		final NewProjectWizard wizard = this;
		IRunnableWithProgress op = new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) throws InvocationTargetException {
				try {
					NewProjectSupport.createProject(name, location, monitor, repo, wizard);
				} catch (CoreException e) {
					throw new InvocationTargetException(e);
				} finally {
					monitor.done();
				}
			}
		};
		try {
			getContainer().run(true, false, op);
		} catch (InterruptedException e) {
			e.printStackTrace();
			return false;
		} catch (InvocationTargetException e) {
			e.printStackTrace();
			Throwable realException = e.getTargetException();
			MessageDialog.openError(getShell(), "Error", realException.getMessage());
			StringWriter sw = new StringWriter();
			realException.printStackTrace(new PrintWriter(sw));
			//String exceptionAsString = sw.toString();
			//MessageDialog.openError(getShell(), "Error", exceptionAsString);
			return false;
		}

		//switches to a new perspective. Should only be used, 
		//when the decide perspective is more useful than the other ones
		//TODO: finish decide perspective
		//BasicNewProjectResourceWizard.updatePerspective(_configurationElement);

		return true;
	}

	
	/**
	 * This method is called when to determine the next wizard page
	 * returns null if there is no next page
	 */
	@Override
	public IWizardPage getNextPage(IWizardPage currentPage) {

		
		if (currentPage == configPage) {
			microservicePage.setMicroservices();
			return microservicePage;
		} else if (currentPage == projectPage) {
			Repository repo = projectPage.getRepository();
			if (!repo.isNewRepo()) {
				configPage.setPageComplete(true);
				if(!canFinish()) {
				getContainer().updateButtons();}
				//System.out.println("Config Page Complete: "+configPage.isPageComplete());
				
				return null;
			}else {
				configPage.setPageComplete(project!=null && !project.getMicroservices().isEmpty());
				
				//System.out.println("Config Page Complete: "+configPage.isPageComplete());
				return configPage;
			}
			
		}
		return null;
	}
	

	/**
	 * We will accept the selection in the workbench to see if we can initialize
	 * from it.
	 * 
	 * @see IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.selection = selection;
		//IPerspectiveRegistry ipr = workbench.getPerspectiveRegistry();
		//for (IPerspectiveDescriptor ipd : ipr.getPerspectives()) {
			//System.out.println("Label: " + ipd.getLabel() + " - ID: " + ipd.getId() + " - Description: " + ipd.getDescription());
		//}
		
	}

	@Override
	public void setInitializationData(IConfigurationElement config, String propertyName, Object data)
			throws CoreException {

		_configurationElement = config;

	}
}