package eu.DECIDEh2020.architect.plugin.layoutComponents;

import java.util.ArrayList;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.wb.swt.SWTResourceManager;
import org.eclipse.swt.widgets.Label;

import eu.DECIDEh2020.appManager.models.AppDescription;
import eu.DECIDEh2020.appManager.models.AvailabilityNfr;
import eu.DECIDEh2020.appManager.models.Microservice;
import eu.DECIDEh2020.appManager.models.Nfr;
import eu.DECIDEh2020.appManager.models.ScalabilityNfr;

//first version of a NFR Composite, not used
public class NfrComposite extends Composite {
	private AppDescription appDescription;
	private Color background = SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND);
	private Button btnStateless;
	private Nfr nfr;
	private java.util.List<ContentChangeListener> contentChangeListeners = new ArrayList<ContentChangeListener>();
	
	public NfrComposite(Composite parent, Nfr nfr, Color background, AppDescription appDescription) {
		super(parent, SWT.None);
		this.background = background;
		setBackground(background);
		this.nfr = nfr;
		this.appDescription = appDescription;
		createContent();
	}

	private void createContent() {
		this.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false, 1, 1));
		this.setLayout(new FillLayout(SWT.HORIZONTAL));
			
		Group grpNfr = new Group(this, SWT.NONE);
		grpNfr.setBackground(background);
		grpNfr.setLayout(new FillLayout(SWT.HORIZONTAL));
		grpNfr.setText(nfr.getType());
		
		Composite composite = new Composite(grpNfr, SWT.NONE);
		composite.setBackground(background);
		composite.setLayout(new FillLayout(SWT.HORIZONTAL));
		
		Composite nfrComposite;
		switch (nfr.getType()) {
		case ("Availability"):
			try {
				nfrComposite = new NfrAvailabilityComposite(composite, (AvailabilityNfr) nfr);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			break;
		case ("Scalability"):
			try {
				nfrComposite = new NfrScalabilityComposite(composite, (ScalabilityNfr) nfr);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			break;
		default:
			nfrComposite = new Composite(composite, SWT.NONE);
			nfrComposite.setLayout(new FillLayout(SWT.HORIZONTAL));

			Label lblNfrHasNo = new Label(nfrComposite, SWT.NONE);
			lblNfrHasNo.setAlignment(SWT.CENTER);
			lblNfrHasNo.setText("NFR has no Type");
		}
	}

	public void addContentChangeListener(ContentChangeListener contentChangedListener) {
		contentChangeListeners.add(contentChangedListener);
	}

	public void setContentChanged() {
		for (ContentChangeListener ccl : contentChangeListeners) {
			ccl.contentChanged();
		}
	}
	
}