/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* Eclipse Public License version 2.0 which accompanies
* this distribution, and is available at
* https://opensource.org/licenses/EPL-2.0
*
* Contributors:
*
* Torben Jastrow (Fraunhofer FOKUS) 
* 
* **Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu */
package eu.DECIDEh2020.architect.patterns.composites;

import java.util.ArrayList;

import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.wb.swt.SWTResourceManager;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;

import eu.DECIDEh2020.appManager.models.AppDescription;
import eu.DECIDEh2020.appManager.models.Nfr;
import eu.DECIDEh2020.appManager.models.Pattern;
import eu.DECIDEh2020.architect.cloudpatterns.Patterns;
import eu.DECIDEh2020.architect.plugin.editor.Page;
import eu.DECIDEh2020.architect.plugin.utils.CloudPatternHelper;

public class PatternPageComposite extends Page {

	private java.util.List<Pattern> patterns;
	private CheckboxTableViewer checkboxTableViewer_basic, checkboxTableViewer_good;
	private PatternDetailComposite patternDetailComposite;
	private Table table_basic, table_good;
	private AppDescription appdescription;

	public class PatternFilter extends ViewerFilter {

		private String filter = "http://decideh2020.eu/resources/patterncategories/fundamental";
		private Boolean show = true;

		public void show(Boolean f) {
			// ensure that the value can be used for matching
			this.show = f;
		}

		
		@Override
		public boolean select(Viewer viewer, Object parentElement, Object element) {
			Pattern p = (Pattern) element;
			return p.getCategories().contains(filter) ? !show : show;
		}
	}

	private static class PatternLabelProvider extends LabelProvider {
		public Image getImage(Object element) {
			return null;
		}

		public String getText(Object element) {
			return ((Pattern) element).getTitle();
		}
	}

	private static class PatternContentProvider implements IStructuredContentProvider {
		@SuppressWarnings("unchecked")
		public Object[] getElements(Object inputElement) {

			return ((java.util.List<Pattern>) inputElement).toArray();

		}

		public void dispose() {
		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
	}

	/**
	 * Create the composite.
	 * 
	 * @param parent
	 * @param style
	 */
	public PatternPageComposite(Composite parent, int style, AppDescription appdescription) {
		super(parent, style, appdescription);
		this.appdescription = appdescription;
		patterns = new ArrayList<>();
		createGUI();

	}

	private void createGUI() {
		setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		setLayout(new GridLayout(1, false));

		CLabel lblDesignPatterns = new CLabel(this, SWT.NONE);
		lblDesignPatterns.setFont(SWTResourceManager.getFont("Segoe UI", 12, SWT.BOLD));
		lblDesignPatterns.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		lblDesignPatterns.setText("Design Patterns");

		Label label = new Label(this, SWT.SEPARATOR | SWT.HORIZONTAL);
		label.setFont(SWTResourceManager.getFont("Segoe UI", 12, SWT.BOLD));
		label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

		Composite composite_1 = new Composite(this, SWT.NONE);
		composite_1.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		composite_1.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		GridLayout gl_composite_1 = new GridLayout(2, true);
		gl_composite_1.horizontalSpacing = 25;
		composite_1.setLayout(gl_composite_1);

		Label lblSuggestedPatterns = new Label(composite_1, SWT.NONE);
		lblSuggestedPatterns.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		lblSuggestedPatterns.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_LIGHT_SHADOW));
		lblSuggestedPatterns.setFont(SWTResourceManager.getFont("Segoe UI", 9, SWT.BOLD));
		lblSuggestedPatterns.setText("Suggested Patterns");

		Label lblPatternDetail = new Label(composite_1, SWT.NONE);
		lblPatternDetail.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_LIGHT_SHADOW));
		lblPatternDetail.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		lblPatternDetail.setFont(SWTResourceManager.getFont("Segoe UI", 9, SWT.BOLD));
		lblPatternDetail.setText("Pattern detail");

		Composite composite_2 = new Composite(composite_1, SWT.NONE);
		composite_2.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		composite_2.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		composite_2.setLayout(new GridLayout(1, false));

		checkboxTableViewer_good = CheckboxTableViewer.newCheckList(composite_2, SWT.BORDER | SWT.FULL_SELECTION);
		table_good = checkboxTableViewer_good.getTable();

		table_good.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		checkboxTableViewer_good.setLabelProvider(new PatternLabelProvider());
		checkboxTableViewer_good.setContentProvider(new PatternContentProvider());
		checkboxTableViewer_good.setInput(patterns);
		PatternFilter noFundamentals = new PatternFilter();
		noFundamentals.show(true);
		checkboxTableViewer_good.setFilters(noFundamentals);

		Label lblbadPatterns = new Label(composite_2, SWT.NONE);
		lblbadPatterns.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		lblbadPatterns.setText("Basic Patterns");
		lblbadPatterns.setFont(SWTResourceManager.getFont("Segoe UI", 9, SWT.BOLD));
		lblbadPatterns.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_LIGHT_SHADOW));

		checkboxTableViewer_basic = CheckboxTableViewer.newCheckList(composite_2, SWT.BORDER | SWT.FULL_SELECTION);
		checkboxTableViewer_basic.addCheckStateListener(new ICheckStateListener() {
			public void checkStateChanged(CheckStateChangedEvent event) {
				Pattern pattern = (Pattern) event.getElement();
				pattern.setSelected(event.getChecked());
			}
		});
		checkboxTableViewer_basic.setAllGrayed(true);
		PatternFilter fundamentals = new PatternFilter();
		fundamentals.show(false);
		checkboxTableViewer_basic.setFilters(fundamentals);

		table_basic = checkboxTableViewer_basic.getTable();

		table_basic.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		checkboxTableViewer_basic.setLabelProvider(new PatternLabelProvider());
		checkboxTableViewer_basic.setContentProvider(new PatternContentProvider());
		checkboxTableViewer_basic.setInput(patterns);

		patternDetailComposite = new PatternDetailComposite(composite_1, SWT.NONE);
		patternDetailComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));

		checkboxTableViewer_good.addCheckStateListener(new ICheckStateListener() {
			public void checkStateChanged(CheckStateChangedEvent event) {
				Pattern pattern = (Pattern) event.getElement();
				pattern.setSelected(event.getChecked());
			}
		});

		table_basic.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				int i = table_basic.getSelectionIndex();
				if (i >= 0) {
					Pattern pattern = (Pattern) checkboxTableViewer_basic.getElementAt(i);
					patternDetailComposite.updatePattern(pattern);

				}
			}
		});
		table_good.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				int i = table_good.getSelectionIndex();
				if (i >= 0) {
					Pattern pattern = (Pattern) checkboxTableViewer_good.getElementAt(i);
					patternDetailComposite.updatePattern(pattern);

				}
			}
		});
		if (table_good.getItemCount() > 0) {
			table_good.select(0);
			patternDetailComposite.updatePattern((Pattern) checkboxTableViewer_good.getElementAt(0));
		}

		recomputeNFRs();
	}

	void recomputeNFRs() {

		updatePatterns(appdescription.getNfrs());
	}

	public void updatePatterns(java.util.List<Nfr> nfrs) {
		patterns = CloudPatternHelper.inferPatterns(nfrs);
		Patterns pat = Patterns.createDefaultPatterns();
		java.util.List<ObjectNode> funds = pat.getPatternsByCategory("fundamental");
		ObjectMapper mapper = new ObjectMapper();

		for (ObjectNode objectNode : funds) {
			try {
				Pattern p = mapper.treeToValue(objectNode, Pattern.class);
				patterns.add(p);
			} catch (JsonProcessingException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		checkboxTableViewer_good.setInput(patterns);
		checkboxTableViewer_basic.setInput(patterns);

		for (Pattern pattern : patterns) {
			Boolean selected = false;
			for (Pattern p : appdescription.getRecommendedPatterns()) {
				if (pattern.getTitle().equals(p.getTitle())) {
					selected = Boolean.logicalOr(selected, p.isSelected());
				}
			}
			pattern.setSelected(selected);
		}

		for (Pattern pattern : patterns) {
			checkboxTableViewer_basic.setChecked(pattern, pattern.isSelected());
			checkboxTableViewer_good.setChecked(pattern, pattern.isSelected());
		}

		appdescription.setRecommendedPatterns(patterns);
		if (table_good.getItemCount() > 0) {
			table_good.select(0);
			patternDetailComposite.updatePattern((Pattern) checkboxTableViewer_good.getElementAt(0));
		} else {
			patternDetailComposite.updatePattern(null);
		}
	}

	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}

	@Override
	public void open(AppDescription appdescription) {
		this.appdescription = appdescription;
		recomputeNFRs();
	}

}
