/*
 * Copyright (c) 2017 Fraunhofer FOKUS.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the
 * GNU AGPL v3 which accompanies
 * this distribution, and is available at
 * http://www.gnu.org/licenses/agpl.txt
 *
 * Contributors:
 * Simon Dutkowski Fraunhofer FOKUS
 *
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 */
package eu.DECIDEh2020.architect.cloudpatterns.compendium;

import eu.DECIDEh2020.architect.cloudpatterns.Patterns;
import eu.DECIDEh2020.architect.cloudpatterns.compendium.service.CloudPatternsService;
import io.vertx.core.AbstractVerticle;
import io.vertx.core.Future;
import io.vertx.serviceproxy.ProxyHelper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class CloudPatternsCompendiumVerticle extends AbstractVerticle {
    private Logger log = LoggerFactory.getLogger(getClass());

    @Override
    public void start(Future<Void> startFuture) throws Exception {
        try {
            Patterns patterns = Patterns.createDefaultPatterns();
            CloudPatternsService patternsService = CloudPatternsService.create(patterns, ready -> {
                if (ready.succeeded()) {
                    ProxyHelper.registerService(CloudPatternsService.class, vertx, ready.result(), "cloudpatterns");
                    startFuture.complete();
                } else {
                    startFuture.fail(ready.cause());
                }
            });
        } catch (Exception e) {
            log.error("creating default patterns", e);
            startFuture.fail(e);
        }
    }

}
