/*
 * Copyright (c) 2017 Fraunhofer FOKUS.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the
 * GNU AGPL v3 which accompanies
 * this distribution, and is available at
 * http://www.gnu.org/licenses/agpl.txt
 *
 * Contributors:
 * Simon Dutkowski Fraunhofer FOKUS
 *
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 */
package eu.DECIDEh2020.architect.cloudpatterns.compendium.service;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import eu.DECIDEh2020.architect.cloudpatterns.DECIDEPatterns;
import eu.DECIDEh2020.architect.cloudpatterns.Patterns;
import eu.DECIDEh2020.architect.cloudpatterns.compendium.converters.NFRConverter;
import eu.DECIDEh2020.architect.cloudpatterns.compendium.converters.PatternConverter;
import io.vertx.core.AsyncResult;
import io.vertx.core.Future;
import io.vertx.core.Handler;
import io.vertx.core.json.Json;
import io.vertx.core.json.JsonArray;
import io.vertx.core.json.JsonObject;
import org.apache.jena.rdf.model.Resource;

import java.util.List;
import java.util.stream.Collectors;

class CloudPatternsServiceImpl implements CloudPatternsService {

    Patterns patterns;

    CloudPatternsServiceImpl(Patterns patterns, Handler<AsyncResult<CloudPatternsService>> readyHandler) {
        this.patterns = patterns;
        readyHandler.handle(Future.succeededFuture(this));
    }

    @Override
    public CloudPatternsService listPatterns(Handler<AsyncResult<JsonArray>> handler) {
        List<Resource> result = patterns.listPatterns();
        handler.handle(Future.succeededFuture(new JsonArray(result.stream().map(PatternConverter::toJson).collect(Collectors.toList()))));
        return this;
    }

    @Override
    public CloudPatternsService listNFRs(Handler<AsyncResult<JsonArray>> handler) {
        List<Resource> result = patterns.listNFRs();
        handler.handle(Future.succeededFuture(new JsonArray(result.stream().map(NFRConverter::toJson).collect(Collectors.toList()))));
        return this;
    }

    @Override
    public CloudPatternsService inferPatterns(JsonArray nfrs, Handler<AsyncResult<JsonArray>> handler) {
        ObjectMapper m = new ObjectMapper();
        List<ObjectNode> nfrList = nfrs.stream().map(nfr -> m.createObjectNode().put("kind", nfr.toString())).collect(Collectors.toList());

        List<ObjectNode> inferredPatterns = patterns.inferPatternsJson(nfrList);
        JsonArray array = new JsonArray(
            inferredPatterns.stream().map(pattern -> new JsonObject(Json.encodePrettily(pattern))).collect(Collectors.toList())
        );

        handler.handle(Future.succeededFuture(array));

        return this;
    }

    @Override
    public CloudPatternsService getPattern(String name, Handler<AsyncResult<JsonObject>> handler) {
        ObjectNode pattern = patterns.getPatternAsJson(DECIDEPatterns.BASE + "patterns/" + name);
        if (pattern != null) {
            handler.handle(Future.succeededFuture(new JsonObject(Json.encodePrettily(pattern))));
        } else {
            handler.handle(Future.failedFuture("Pattern not found."));
        }
        return this;
    }

}
